# -*- coding: utf-8 -*-

# Imports
from .database import Database
from .renderer import render_menu
from .helpers import clear_window
from .inquirer import ask_the_word
from .renderer import render_banner
from .inquirer import ask_the_level
from .inquirer import ask_the_letter
from .helpers import find_occurrences
from .renderer import render_game_over
from .renderer import render_scoreboard
from .renderer import render_placeholder
from .renderer import render_invalid_level

# Game levels
from .easy_level import check_round_state as easy_level_state_checker
from .easy_level import generate_placeholder as easy_level_placeholder
from .hard_level import check_round_state as hard_level_state_checker
from .hard_level import generate_placeholder as hard_level_placeholder

# Initialize database
database_client = Database()


def check_letter(hunches, word, letter):
    print("")
    if len(find_occurrences(hunches, letter)) >= 2:
        print("    >> The letter %s has already been used in a previous round." % letter)
    if letter not in word:
        print("    >> The letter %s is not part of the word!" % letter)


def update_score(db, word, letter):
    if db.get_score() < 1:
        return
    if letter not in word:
        db.deduct_score()


def start_round(db, already_started):
    if not already_started:
        # Set word
        db.set_word(ask_the_word())
        # Set score
        db.set_score()
    # Increase round every call
    if db.get_round() < len(db.get_word()):
        db.increase_round()
    # Render scoreboard
    render_scoreboard(db.get_level(), db.get_round(), db.get_score())
    # Render placeholder
    if db.get_level() == "1":
        render_placeholder(easy_level_placeholder(
            db, db.get_word(), db.get_letter()))
    else:
        render_placeholder(hard_level_placeholder(
            db, db.get_word(), db.get_letter()))
    # Check round state
    if db.get_level() == "1":
        easy_level_state_checker(db)
    else:
        hard_level_state_checker(db)
    # Check letter
    check_letter(db.get_hunches(), db.get_word(), db.get_letter())
    # Set letter
    db.set_letter(ask_the_letter())
    # Add letter to hunches
    db.add_hunch(db.get_letter())
    # Update score
    update_score(db, db.get_word(), db.get_letter())
    # Apply recursion
    start_round(db, True)


def start_game(show_banner):
    if show_banner:
        render_banner()
    render_menu()
    # Set the game level
    database_client.set_level(ask_the_level())
    # Game levels
    if database_client.get_level() == "1":
        clear_window()
        render_banner()
        start_round(database_client, False)
    elif database_client.get_level() == "2":
        clear_window()
        render_banner()
        start_round(database_client, False)
    else:
        clear_window()
        render_banner()
        render_invalid_level()
        start_game(False)



# -*- coding: utf-8 -*-

# Imports
from .helpers import print_line
from .helpers import clear_window
from .renderer import render_banner


def ask_the_level():
    return input("    Choose the level: ")


def ask_the_word():
    print("")
    print_line()
    print("")
    chosen_word = input(
        "    PLAYER 1> Choose a word with at least 7 letters: ").upper().strip()
    if len(chosen_word) < 7:
        clear_window()
        render_banner()
        print("")
        print_line()
        print("")
        print("    You need to write a word with at least 7 letters. Try again!")
        ask_the_word()
    return chosen_word


def ask_the_letter():
    print_line()
    print("")
    return input("    PLAYER 2> Enter a letter: ").upper().strip()


def ask_final_guess():
    print("")
    print_line()
    print("")
    return input("    PLAYER 2> What is the word? ").upper().strip()



# -*- coding: utf-8 -*-

# Imports
import os


def print_line():
    print("    ----------------------------------------------------------------------")


def print_message(message):
    print_line()
    print("")
    print("    %s" % message)
    print("")
    print_line()
    print("")


def clear_window():
    os.system('cls' if os.name == 'nt' else 'clear')


def find_occurrences(string, character):
    return [i for i, letter in enumerate(string) if letter == character]



# -*- coding: utf-8 -*-

# Imports
from .engine import start_game
from .helpers import print_message


def main():
    # Start the game
    try:
        start_game(True)
    # Catch exceptions
    except Exception as e:
        print_message("An error has occurred during the game: %s" % e)
    except KeyboardInterrupt:
        print("")
        print("\n    Game closed. See you later :)")
        print("")
        exit(0)


# Call main
main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="hangman-game",
    version="1.0.5",
    author="Christopher Enytc",
    author_email="chris@enytc.com",
    description="A CLI game built with Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/chrisenytc/hangman",
    packages=setuptools.find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'hangman = hangman_game.__main__:main'
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding: utf-8 -*-

# Imports
from .helpers import find_occurrences
from .renderer import render_game_over


def generate_placeholder(db, word, letter):
    placeholder = ""
    for index in range(0, len(word)):
        placeholder = placeholder + "__ "
    if len(db.get_placeholders()) == 0:
        db.set_placeholders(placeholder.split())
    occurrences = find_occurrences(word, letter)
    for index in occurrences:
        db.update_placeholder(index, letter)
    return " ".join(db.get_placeholders())


def check_round_state(db):
    if db.get_score() == 0:
        render_game_over(1, db.get_word())
        exit(0)
    if db.get_word() == "".join(db.get_placeholders()):
        render_game_over(2, db.get_word())
        exit(0)
